//+------------------------------------------------------------------+
//|                                             EGEN SAR Advisor.mq4 |
//|                        Copyright 2013, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

extern bool Buy   = true;
extern bool Sell  = true;
extern int  Lots  = 1;

double SAR;

double DOUBLE(double price)
{
   return(NormalizeDouble(price,Digits));
}

double SAR(int shift)
{
   return(DOUBLE(iSAR(NULL,0,0.02,0.2,shift)));
}

double H(int shift)
{
   return(iHigh(NULL,0,shift));
}

double L(int shift)
{
   return(iLow(NULL,0,shift));
}

bool BARS(int shift)
{
   return(shift<iBars(NULL,0));
}

double INFO(int type)
{
   return(MarketInfo(Symbol(),type));
}

int SPREAD()
{
   return(INFO(MODE_SPREAD));
}

int STOPLEVEL()
{
   return(INFO(MODE_STOPLEVEL));
}

double MINLOT()
{
   return(INFO(MODE_MINLOT));
}

double LEVEL()
{
   return(STOPLEVEL()*Point);
}

double ASK()
{
   return(Ask+LEVEL());
}

double BID()
{
   return(Bid-LEVEL());
}

double LOTS()
{
   double LT=MINLOT();
   if(Lots<1) return(LT); 
   return(LT*Lots);
}

bool TOTAL(int index)
{
   return(index<OrdersTotal());
}

bool SelectOrder(int index)
{
   return(OrderSelect(index,SELECT_BY_POS));
}

double OrderPrice()
{
   return(DOUBLE(OrderOpenPrice()));
}

void CloseOrder(double price)
{
   OrderClose(OrderTicket(),OrderLots(),price,SPREAD());
}

void DeleteOrder()
{
   OrderDelete(OrderTicket());
}

void ModifyOrder()
{
   OrderModify(OrderTicket(),SAR,0,0,0);
}

void BuyStop()
{
   OrderSend(Symbol(),OP_BUYSTOP,LOTS(),SAR,SPREAD(),0,0);
}

void BuyLimit()
{
   OrderSend(Symbol(),OP_BUYLIMIT,LOTS(),SAR,SPREAD(),0,0);
}

void SellStop()
{
   OrderSend(Symbol(),OP_SELLSTOP,LOTS(),SAR,SPREAD(),0,0);
}

void SellLimit()
{
   OrderSend(Symbol(),OP_SELLLIMIT,LOTS(),SAR,SPREAD(),0,0);
}

bool OP(int op)
{
   return(OrderType()==op);
}

bool SELECT(int index)
{
   if(SelectOrder(index))
      if(OrderSymbol()==Symbol()) return(true);
   return(false);      
}

int ORDER()
{
   if(OP(OP_BUY))
      return(+1); else
   if(OP(OP_SELL))
      return(-1); else
   return(0);      
}

int STOP()
{
   if(OP(OP_BUYSTOP))
      return(+1); else
   if(OP(OP_SELLSTOP))
      return(-1); else
   return(0);      
}

int LIMIT()
{
   if(OP(OP_BUYLIMIT))
      return(+1); else
   if(OP(OP_SELLLIMIT))
      return(-1); else
   return(0);      
}

void CLOSEBUY(int index)
{
   if(SELECT(index))   
   if(ORDER()>0) 
      CloseOrder(Bid); else  
   if(STOP()>0)
      DeleteOrder(); else       
   if(LIMIT()>0)
      DeleteOrder();
   else return;         
}

void CLOSESELL(int index)
{
   if(SELECT(index))   
   if(ORDER()<0) 
      CloseOrder(Ask); else  
   if(STOP()<0)
      DeleteOrder(); else       
   if(LIMIT()<0)
      DeleteOrder();
   else return;         
}

void CLOSEBUYS()
{
   for(int i=0; TOTAL(i); i++) CLOSEBUY(i);
}

void CLOSESELLS()
{
   for(int i=0; TOTAL(i); i++) CLOSESELL(i);
}

bool BUYS()
{
   if(Buy) return(false);
   CLOSEBUYS();
   return(true);
}

bool SELLS()
{
   if(Sell) return(false);
   CLOSESELLS();
   return(true);
}

bool PRICE()
{
   if(OrderPrice()>SAR) return(false);
   if(OrderPrice()<SAR) return(false);  
   return(true);
}

bool EXIST()
{
   for(int i=0; TOTAL(i); i++)
      if(SELECT(i))
         if(PRICE()) return(true);
      else continue;        
   return(false);      
}

bool STOPBUY()
{
   if(STOP()>0)
      if(OrderPrice()>SAR) return(true);
   return(false);      
}

bool SelectBuyStop()
{
   for(int i=0; TOTAL(i); i++)
      if(SELECT(i))
         if(STOPBUY()) return(true);
      else continue;        
   return(false);      
}

bool LIMITBUY()
{
   if(LIMIT()>0)
      if(OrderPrice()<SAR) return(true);
   return(false);      
}

bool SelectBuyLimit()
{
   for(int i=0; TOTAL(i); i++)
      if(SELECT(i))
         if(LIMITBUY()) return(true);
      else continue;        
   return(false);      
}

bool STOPSELL()
{
   if(STOP()<0)
      if(OrderPrice()<SAR) return(true);
   return(false);      
}

bool SelectSellStop()
{
   for(int i=0; TOTAL(i); i++)
      if(SELECT(i))
         if(STOPSELL()) return(true);
      else continue;        
   return(false);      
}

bool LIMITSELL()
{
   if(LIMIT()<0)
      if(OrderPrice()>SAR) return(true);
   return(false);      
}

bool SelectSellLimit()
{
   for(int i=0; TOTAL(i); i++)
      if(SELECT(i))
         if(LIMITSELL()) return(true);
      else continue;        
   return(false);      
}

bool BUYSTOP(int shift)
{
   if(EXIST()) return(false);
   if(H(shift)>SAR) return(false);   
   if(SAR<ASK()) return(true);
   if(SelectBuyStop())
      ModifyOrder(); else BuyStop();      
   return(false);      
}

bool BUYLIMIT(int shift)
{
   if(EXIST()) return(false);
   if(L(shift)<SAR) return(false);   
   if(SAR>BID()) return(true);
   if(SelectBuyLimit())
      ModifyOrder(); else BuyLimit();      
   return(false);      
}

bool SELLSTOP(int shift)
{
   if(EXIST()) return(false);
   if(L(shift)<SAR) return(false);   
   if(SAR>BID()) return(true);
   if(SelectSellStop())
      ModifyOrder(); else SellStop();      
   return(false);      
}

bool SELLLIMIT(int shift)
{
   if(EXIST()) return(false);
   if(H(shift)>SAR) return(false);   
   if(SAR<ASK()) return(true);
   if(SelectSellLimit())
      ModifyOrder(); else SellLimit();      
   return(false);      
}

bool BUY(int shift)
{
   if(BUYS())
      return(false); else
   if(BUYSTOP(shift))
      return(true); else
   if(BUYLIMIT(shift))
      return(true); else
   return(false);               
}

bool SELL(int shift)
{
   if(SELLS())
      return(false); else
   if(SELLSTOP(shift))
      return(true); else
   if(SELLLIMIT(shift))
      return(true); else
   return(false);               
}

bool SEND(int shift)
{
   if(BUY(shift)) 
      return(true); else
   if(SELL(shift))
      return(true); else
   return(false);            
}

bool TRADE(int shift)
{
   SAR=SAR(shift); 
   if(SEND(shift)) return(true);
   return(false);
}

void start()
{
   for(int i=0; BARS(i); i++) if(TRADE(i)) break;
}



